/*
 * Copyright (C) 2004 by kabalNET.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding KNCwebserver MUST remain intact.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * @author Brett Haas
 */
package com.kabalnet.kncwebserver.core;

import java.util.ArrayList;
import java.util.List;

public class ThreadPool implements Runnable {
    private TaskQueue queue;

    /**
     * Init Object using default wait timeout
     *
     * @param poolSize Number of worker threads
     */
    public ThreadPool(int poolSize) {
        this(poolSize, -1);
    }

    /**
     * Init Ojbect
     *
     * @param poolSize Number of worker threads
     * @param waitTimeout Time in millis for thread to idle before trying for a task again
     */
    public ThreadPool(int poolSize, long waitTimeout) {
        if (waitTimeout > -1) {
            queue = new TaskQueue(waitTimeout);
        } else {
            queue = new TaskQueue();
        }

        for (int i = 0; i < poolSize; i++) {
            new Thread(this).start();
        }
    }

    /**
     * Place a new task in the queue
     *
     * @param task Runnable
     */
    public void putTask(Runnable task) {
        queue.add(task);
    }

    public void run() {
        while (true) {
            Runnable task = queue.get();

            if (task != null) {
                try {
                    task.run();
                } catch (Exception ex) {
                    ;
                }
            }
        }
    }

    private class TaskQueue {
        private static final long WAIT_TIMEOUT = 0L;
        private List list = new ArrayList();
        private long waitTimeout;

        public TaskQueue() {
            this(WAIT_TIMEOUT);
        }

        public TaskQueue(long waitTimeout) {
            this.waitTimeout = waitTimeout;
        }

        public synchronized void add(Runnable task) {
            list.add(task);
            notifyAll();
        }

        public synchronized Runnable get() {
            while (true) {
                Runnable task = null;

                if (!list.isEmpty()) {
                    task = (Runnable) list.remove(0);
                }

                if (task != null) {
                    return task;
                }

                try {
                    wait(waitTimeout);
                } catch (InterruptedException ex) {
                    ;
                }
            }
        }
    }
}
