/* Copyright (C) 2003 Albert Seward
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sf.jlink;

import java.net.*;
import java.util.*;
import java.io.*;


public class ClientHandler extends Thread {
    private Socket socket;
    private PrintStream out;
    private BufferedReader in;
    private FileSystem fs;
    private String playerID = null;
    private String nowPlaying = null;
    private JWindow gui;
    private JConfig config;
   
    public ClientHandler(Socket socket, FileSystem fs, JWindow gui,JConfig config) throws Exception {
        this.socket = socket;
        this.fs = fs;
        this.gui = gui;
        this.config = config;
        in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
        out = new PrintStream(socket.getOutputStream(), true);
    }
    
    
    public void run() {
        char inBuffer[];
        if(gui != null)
            gui.addPlayer(this);
        while(true) {
            try {
                inBuffer = new char[512];
                if(in.read(inBuffer) < 0)
                    break;
                parseCommand(String.valueOf(inBuffer).trim().replace('\\', '/'));
            } catch(DirectoryNotFoundException e) {
                displayMessage("You need to mount a filetree here","INTERNAL:404.jpg");
                try { socket.close(); } catch(IOException f) {}
                break;
            } catch(FileNotFoundException e) {
                out.print("404");
                out.flush();
            } catch(IOException e) {
                try { socket.close(); } catch(IOException f) {}
                break;
            }
        }
        if(gui != null)
            gui.removePlayer(this);
    }
    
    public void skip() {
        try {
            socket.close();
        } catch(Exception e) {};
    }
    
    public String getNowPlaying() {
        return nowPlaying;
    }
    
    public String getNowPlayingFileName() {
        if(nowPlaying == null)
            return null;
        return(nowPlaying.substring(nowPlaying.lastIndexOf('/')+1));
    }
    
    public String getPlayerID() {
        return playerID;
    }
    
    public InetAddress getPlayerIP() {
        return socket.getInetAddress();
    }
    
    private void parseCommand(String commandLine) throws IOException, FileNotFoundException, DirectoryNotFoundException {
        String command[] = commandLine.split("\\|");
        
        //if(!command[0].startsWith("GET"))
        JLink.debug("Got Command: \"" + commandLine + "\"");
        
        if(command[0].startsWith("LIST")) {
            
            StringBuffer buf = new StringBuffer(250);
            Vector list=null;
            Vector sortedList=new Vector();
            Vector sortedListRep=new Vector();
            String path="";
            if ((commandLine.indexOf("VIDEO")!=-1) && (config.getSubTV()==true)  ) {
                if (commandLine.indexOf("@HELL")==-1) {
                    String MenuName = "[Hell Link Option]";
                    buf.append(MenuName + "| " + "HLOPTION@HELL" + "|1|\n");
                    
                    path = command[0].substring(5).trim() + (command.length > 1?(FileSystem.pathSeparatorChar + command[1].trim()):"");
                    list = fs.list(path);
                    
                }
                else {
                    
                    
                    if (commandLine.indexOf("HLODON@HELL")!=-1) {
                        config.setExtDisplay(3,true);
                    }
                    else if (commandLine.indexOf("HLODOFF@HELL")!=-1) {
                        config.setExtDisplay(3,false);
                    }
              
                    else if (commandLine.indexOf("HLOD-500@HELL")!=-1) {
                        config.setSubDelay( String.valueOf((Integer.parseInt(config.getSubDelay())-500)) );
                    }
                    else if (commandLine.indexOf("HLOD-200@HELL")!=-1) {
                        config.setSubDelay( String.valueOf((Integer.parseInt(config.getSubDelay())-200)) );
                    }
                    else if (commandLine.indexOf("HLOD-100@HELL")!=-1) {
                        config.setSubDelay( String.valueOf((Integer.parseInt(config.getSubDelay())-100)) );
                    }
                    else if (commandLine.indexOf("HLOD-50@HELL")!=-1) {
                        config.setSubDelay( String.valueOf((Integer.parseInt(config.getSubDelay())-50)) );
                    }
                    else if (commandLine.indexOf("HLOD0@HELL")!=-1) {
                        config.setSubDelay("0");
                    }
                    else if (commandLine.indexOf("HLOD+50@HELL")!=-1) {
                        config.setSubDelay( String.valueOf((Integer.parseInt(config.getSubDelay())+50)) );
                    }
                    else if (commandLine.indexOf("HLOD+100@HELL")!=-1) {
                        config.setSubDelay( String.valueOf((Integer.parseInt(config.getSubDelay())+100)) );
                    }
                    else if (commandLine.indexOf("HLOD+200@HELL")!=-1) {
                        config.setSubDelay( String.valueOf((Integer.parseInt(config.getSubDelay())+200)) );
                    }
                    else if (commandLine.indexOf("HLOD+500@HELL")!=-1) {
                        config.setSubDelay( String.valueOf((Integer.parseInt(config.getSubDelay())+500)) );
                    }
                    
                    
                    if (commandLine.indexOf("HLOD")!=-1)
                        config.save();
                    
                    
                    //curent delay status
                    if (config.getExtDisplay(3)==true){
                        String MenuName = "[Current delay state is ON,Delay="+config.getSubDelay() +"ms]";
                        buf.append(MenuName + "| "  +"" + "|2|\n");
                    }
                    else {
                        String MenuName = "[Current delay state is OFF]";
                        buf.append(MenuName + "| "  +"" + "|2|\n");
                        
                    }
                    if (commandLine.indexOf("HLOD")==-1) {//display options
                        
                        String MenuName = "[Delay ON]";
                        buf.append(MenuName + "| "  +"HLODON@HELL" + "|1|\n");
                        
                        MenuName = "[Delay OFF]";
                        buf.append(MenuName + "| "  +"HLODOFF@HELL" + "|1|\n");
                     
                        MenuName = "[-500ms]";
                        buf.append(MenuName + "| "  +"HLOD-500@HELL" + "|1|\n");
                        
                        MenuName = "[-200ms]";
                        buf.append(MenuName + "| "  +"HLOD-200@HELL" + "|1|\n");
                        
                        MenuName = "[-100ms]";
                        buf.append(MenuName + "| "  +"HLOD-100@HELL" + "|1|\n");
                        
                        MenuName = "[-50ms]";
                        buf.append(MenuName + "| "  +"HLOD-50@HELL" + "|1|\n");
                        
                        MenuName = "[Reset to 0ms]";
                        buf.append(MenuName + "| "  +"HLOD0@HELL" + "|1|\n");
                        
                        MenuName = "[+50ms]";
                        buf.append(MenuName + "| "  +"HLOD+50@HELL" + "|1|\n");
                        
                        MenuName = "[+100ms]";
                        buf.append(MenuName + "| "  +"HLOD+100@HELL" + "|1|\n");
                        
                        MenuName = "[+200ms]";
                        buf.append(MenuName + "| "  +"HLOD+200@HELL" + "|1|\n");
                        
                        MenuName = "[+500ms]";
                        buf.append(MenuName + "| "  +"HLOD+500@HELL" + "|1|\n");
                        
                        
                        
                    }
                }
            }
            else {
                path = command[0].substring(5).trim() + (command.length > 1?(FileSystem.pathSeparatorChar + command[1].trim()):"");
                list = fs.list(path);
            }
            
            
            if (commandLine.indexOf("HLOPTION@HELL")==-1) {
	        // correction by oldman feb 2005 to make playlists not sorted
		if (commandLine.toLowerCase().indexOf(".pls")>-1||commandLine.toLowerCase().indexOf(".m3u")>-1){
		                   sortedList=list;
		}else
		{
                if (config.getClean()==true) //clean all file to sort result
                {
                    
                    for(Iterator i = list.iterator(); i.hasNext();) {
                        String name = (String)i.next();
                        if(name.endsWith(FileSystem.pathSeparatorChar)) {//not a file
                            sortedListRep.add(name);
                        }
                        else sortedList.add(CleanupFilename(name)+"#SEP@HELL#"+name);
                    }
                    
                    Collections.sort(sortedList,String.CASE_INSENSITIVE_ORDER);
                    Collections.sort(sortedListRep,String.CASE_INSENSITIVE_ORDER);
                    int pos=0;
                    for(Iterator i = sortedListRep.iterator(); i.hasNext();) {
                        String name = (String)i.next();
                        sortedList.add(pos, name);
                        pos++;
                    }
                }
                else {
                    sortedList=list;
                }
		}
                int pos=-1;
                for(Iterator i = sortedList.iterator(); i.hasNext();) {
                    pos=pos+1;
                    String displayName ="";
                    String name ="";
                    
                    String[] nameSplit= ((String)i.next()).split("#SEP@HELL#");
                    if (nameSplit.length==1){
                        displayName = nameSplit[0];
                        name = nameSplit[0];
                    }
                    else {
                        displayName = nameSplit[0];
                        name = nameSplit[1];
                    }
                    
                    
                    
                    if ( path.indexOf("VIDEO") !=-1) {
                        
                        if(name.endsWith(FileSystem.pathSeparatorChar)) {
                            name = name.substring(0, name.length()-1);
                            buf.append(name + "| " + name + "|1|\n");
                        } else {
                            
                            
                            String confExt=config.getExtFile(0);
                            String[] ext =confExt.split(";");
                            boolean found=false;
                            for (int j=0;(j<ext.length)&&(found==false);j++) {
                                if (name.toLowerCase().endsWith(ext[j])) {
                                    
                                    
                                    if (config.getExtDisplay(0)==false) displayName=displayName.replaceFirst(ext[j],"");
                                    buf.append(displayName + "| " + path + FileSystem.pathSeparatorChar + name + "|0|\n");
                                    found=true;
                                }
                                if(name.toLowerCase().endsWith(".m2v") || name.toLowerCase().endsWith(".m2p")) {
                                    
                                    
                                    if (config.getExtDisplay(0)==false) displayName=displayName.replaceFirst(ext[j],"");
                                    buf.append(displayName + "| " + path + FileSystem.pathSeparatorChar + name.substring(0, name.length()-4) + ".mpeg|0|\n");
                                }
                                
                            }
                            
                            
                        }
                    }
                    else if ( path.indexOf("AUDIO") !=-1) {
                        
                        if(name.endsWith(FileSystem.pathSeparatorChar)) {
                            name = name.substring(0, name.length()-1);
                            buf.append(name + "| " + name + "|1|\n");
                        } else {
                            String confExt=config.getExtFile(1);
                            String[] ext =confExt.split(";");
                            boolean found=false;
                            for (int j=0;(j<ext.length)&&(found==false);j++) {
                                if (name.toLowerCase().endsWith(ext[j])) {
                                    
                                    
                                    if (config.getExtDisplay(1)==false) displayName=displayName.replaceFirst(ext[j],"");
                                    buf.append(displayName + "| " + path + FileSystem.pathSeparatorChar + name + "|0|\n");
                                    found=true;
                                }
                            }
                        }
                    }
                    
                    else if ( path.indexOf("PICTURE") !=-1) {
                        
                        if(name.endsWith(FileSystem.pathSeparatorChar)) {
                            name = name.substring(0, name.length()-1);
                            buf.append(name + "| " + name + "|1|\n");
                        } else {
                            String confExt=config.getExtFile(2);
                            String[] ext =confExt.split(";");
                            boolean found=false;
                            for (int j=0;(j<ext.length)&&(found==false);j++) {
                                if (name.toLowerCase().endsWith(ext[j])) {
                                    
                                    if (config.getExtDisplay(2)==false) displayName=displayName.replaceFirst(ext[j],"");
                                    buf.append(displayName + "| " + path + FileSystem.pathSeparatorChar + name + "|0|\n");
                                    found=true;
                                }
                            }
                        }
                    }
                    
                    
                }
            }
            buf.append("EOL\n");
            String kalle;
            out.write(buf.toString().replace('/', '\\').getBytes());
            out.flush();
            socket.close();
            
        } else if(command[0].startsWith("SIZE")) {
            String filename = command[0].substring(5);
            File file = fs.getFile(filename);
            String filesize = (new Long(file.length())).toString();
            out.print("000000000000000".concat(filesize).substring(filesize.length()));
            out.flush();
            
        }  else if(command[0].startsWith("ACTION")) {
            String filename;
            if(command[0].startsWith("ACTION 1")) {
                filename = command[2].trim();
                playerID = command[1].trim();
            } else {
                filename = command[0].substring(9);
            }
            fs.getFile(filename);
            nowPlaying = filename.trim();
            if(gui != null) {
                gui.removePlayer(this);
                gui.addPlayer(this);
            }
            out.print("200");
            out.flush();
            
        } else if(command[0].startsWith("GET")) {
            
            
            
            boolean close = false;
            String args[] = command[1].trim().split(" ");
            long offset = Long.parseLong(args[0]);
            long length = Long.parseLong(args[1]);
            
            String AddSubExt="";
            if (config.getExtDisplay(3)) //sub delay apply
            {
                //get sub extension
                String confExt=config.getExtFile(3);
                String[] ext =confExt.split(";");
                boolean subFile=false;
                for(int i=0;i<ext.length;i++) {
                    subFile= (command[0].substring(4).indexOf(ext[i])!=-1);
                    if (subFile) break;
                }
                if (subFile) 
                {//sub file adjust delay
                    if (offset==0) 
                    {
                        //Process p=null;
                        try{
                            Runtime.getRuntime().exec(config.getSubProg()+" \"/DELAY("+fs.getFile(command[0].substring(4))+"/"+fs.getFile(command[0].substring(4))+".KISS/0/"+config.getSubDelay()+")\" ");
                        }
                        catch (Exception ex) {
                            JLink.error("Exception while delaying subtitle : (" + ex.getLocalizedMessage() + ") Check Urusoft subtitle workshop path !\n", null, false);
                        }
                        
                    }
                     
                    
                       AddSubExt=".KISS";
                }
                
            }
            
        
            RandomAccessFile file=null;
            try
            {file = new RandomAccessFile(fs.getFile(command[0].substring(4))+AddSubExt,"r");
            }
            catch (Exception ex) {   //read normal subtitle file if file access problem
                file = new RandomAccessFile(fs.getFile(command[0].substring(4)),"r");
            }
            file.seek(offset);
            
            if(length == 0) {
                length = file.length()-offset;
                close = true;
            }
            
            for(; length > 0; ) {
                //int send = 32768;
                int send = new Integer(config.getBufferSize()).intValue(); //config buffer size
                if(length < send)
                    send = (int)length;
                length -= send;
                byte buffer[] = new byte[send];
                file.read(buffer);
                out.write(buffer, 0, send);
                out.flush();
                if(!socket.isConnected())
                    return;
            }
            file.close();
            if(close)
                socket.close();
            
            
        } else if(!commandLine.equals("")) {
            JLink.error("Unsupported command (" + commandLine + ")! Please report this BUG\n", null, false);
        }
    }
    
    private void displayMessage(String msg, String file) {
        out.println(msg + "| " + file + "|0|");
    }
    
    public String removeCharFromString(String s, char c) {
        StringBuffer sb=new StringBuffer(s);
        int index=s.length();
        while (index > 0) {
            index=s.lastIndexOf(c,index);
            if (index >=0) sb.deleteCharAt(index);
        } return sb.toString();
    }
    
    public String replaceString( String s, String f, String r ) {
        if (s == null)  return s;
        if (f == null)  return s;
        if (r == null)  r = "";
        
        int index01 = s.indexOf( f );
        while (index01 != -1) {
            s = s.substring(0,index01) + r + s.substring(index01+f.length());
            index01 += r.length();
            index01 = s.indexOf( f, index01 );
        }
        return s;
    }
    
    public String CleanupFilename(String filename) { //adapted from emule 0.30e source code
        
        String tempStr;
        filename=filename.toLowerCase();
        
        
        // Replace . with Spaces - except the last one (extention-dot)
        int extpos=filename.lastIndexOf('.');
        if (extpos>=0) {
            for (int i=0;i<extpos;++i) {
                if (filename.charAt(i)!='.') continue;
                
                if (i>0 && i<filename.length()-1 && Character.isDigit(filename.charAt(i-1)) && Character.isDigit(filename.charAt(i+1)) ) continue;
                
                char[] tmp = filename.toCharArray();
                tmp[i]=' ';
                filename=String.valueOf(tmp);
                
                
            }
        }
        
        
        
        // Replace Space-holders with Spaces
        filename=filename.replace('_',' ');
        filename=filename.replace('+',' '); //SyruS for Jigle
        
        
        //remove user substrings
        String[] resToken;
        String strlink=config.getCleanStrings();
        
        resToken= strlink.toLowerCase().split(";");
        for (int i=0;i<resToken.length;i++) {
            filename=replaceString(filename,resToken[i],"");
        }
        
        
        //SyruS additional cleanup
        // invalid for filenames
        filename=removeCharFromString(filename,'\\'); //[edit: oops, one \ was missing]
        filename=removeCharFromString(filename,'\"');
        filename=removeCharFromString(filename,'/');
        filename=removeCharFromString(filename,':');
        filename=removeCharFromString(filename,'*');
        filename=removeCharFromString(filename,'?');
        filename=removeCharFromString(filename,'<');
        filename=removeCharFromString(filename,'>');
        filename=removeCharFromString(filename,'|');
        // other common nonsense (u can use dots here!)
        filename=removeCharFromString(filename,'=');
        
        int pos1,pos2;
        pos1=-1;
        
        while (true) {
            pos1=filename.indexOf('[',pos1+1);
            if (pos1==-1) break;
            pos2=filename.indexOf(']',pos1);
            if (pos1>-1 && pos2>pos1) {
                if (pos2-pos1 > 1) {
                    tempStr=filename.substring(pos1+1,(pos1+1)+pos2-pos1-1);
                    int numcount=0;
                    for (int i=0;i<tempStr.length();++i)
                        if (Character.isDigit(tempStr.charAt(i))) ++numcount;
                    if (numcount>tempStr.length()/2) continue;
                }
                filename=filename.substring(0,pos1)+filename.substring(pos2+1,filename.length());
                pos1-=(pos2-pos1);
            } else break;
        }
        
        // Barry - Some additional formatting
        filename=replaceString(filename,"()", "");
        filename=replaceString(filename,"  ", " ");
        filename=replaceString(filename," .",".");
        
        filename=replaceString(filename,"( ", "(");
        filename=replaceString(filename," )", ")");
        filename=replaceString(filename,"()", "");
        filename=replaceString(filename,"{ ", "{");
        filename=replaceString(filename," }", "}");
        filename=replaceString(filename,"{}", "");
        
        // Make leading Caps
        if (filename.length()>1) {
            tempStr=String.valueOf(filename.charAt(0));
            tempStr=tempStr.toUpperCase();
            char[] tmp = filename.toCharArray();
            tmp[0]=tempStr.charAt(0);
            filename=String.valueOf(tmp);
            
            
            int topos=filename.lastIndexOf('.')-1;
            if (topos<0) topos=filename.length()-1;
            
            for (int ix=0; ix<topos; ix++) {
                if (!Character.isLetter(filename.charAt(ix))) //if (filename.GetAt(ix) == ' ' || isdigit(filename.GetAt(ix)) )
                {
                    if (ix<filename.length()-2 && Character.isDigit(filename.charAt(ix+2) ))
                        continue;
                    
                    tempStr=String.valueOf(filename.charAt(ix+1));
                    tempStr=tempStr.toUpperCase();
                    tmp = filename.toCharArray();
                    tmp[ix+1]=tempStr.charAt(0);
                    filename=String.valueOf(tmp);
                    
                    
                }
            }
        }
        filename=filename.trim();
        extpos=filename.lastIndexOf('.');
        if (extpos>0) {
            String beforeExt=filename.substring(0,extpos);
            beforeExt=beforeExt.trim();
            String afterExt=filename.substring(extpos);
            if (beforeExt.charAt(beforeExt.length()-1)=='-') {
                char[] tmp = beforeExt.toCharArray();
                tmp[beforeExt.length()-1]=' ';
                beforeExt=String.valueOf(tmp);
                beforeExt=beforeExt.trim();
            }
            if (beforeExt.charAt(0)=='-') {
                char[] tmp = beforeExt.toCharArray();
                tmp[0]=' ';
                beforeExt=String.valueOf(tmp);
                beforeExt=beforeExt.trim();
            }
            filename=beforeExt.concat(afterExt);
        }
        
        filename=replaceString(filename,"- ", "-");
        filename=replaceString(filename," -", "-");
        return filename;
    }
    
    
}

