/* Copyright (C) 2003 Albert Seward
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Correction is done in the code by Jrgen stensen (oldman) feb 2005. The correction
 *  is linked up with the corrections that are done in the PlayList class.
 *
 */

package sf.jlink;

import java.io.*;
import java.util.*;

public class Mount {
    protected String mapPath;
    protected String mountPoint;
    public boolean supportPLS = false;
    public boolean supportM3U = false;
    
    public Mount() {
        try {
            setMountPoint("");
            setMapPath("");
        } catch(DirectoryNotFoundException e) {}
    }
    
    public Mount(String mountPoint, String path) throws DirectoryNotFoundException {
        setMountPoint(mountPoint);
        setMapPath(path);
    }
    
    public String getMountPoint() {
        return mountPoint;
    }
    
    public void setMountPoint(String path) {
        mountPoint = normalizePath(path);
    }
    
    public void setMapPath(String path) throws DirectoryNotFoundException {
        File file = new File(path);
         path = file.getAbsolutePath();
         if(!file.canRead())
            throw new DirectoryNotFoundException(path);
         mapPath = path;
    }
    
    public String getMapPath() {
        return mapPath;
    }
        
    public File get(String path) {
        path = normalizePath(path);
        if(!path.startsWith(mountPoint))
            return null;
        path = path.substring(mountPoint.length());
        
        File f = new File(mapPath + path); 
        JLink.debug("Mount path:"+mapPath+" "+path);
        if(f.isFile() && f.canRead())
            return f;
        
        // Ugly fix for .m2v .m2p support 
        if(path.toLowerCase().endsWith(".mpeg")) {
            path = path.substring(0, path.length()-5);
            f = new File(mapPath + path + ".m2v"); 
            if(f.isFile() && f.canRead())
                return f;
            f = new File(mapPath + path + ".m2p"); 
            if(f.isFile() && f.canRead())
                return f;
        }
        
        return PlayList.getFile(mapPath + path);
    }
    
    public Vector list(String path) {
        Vector ret = new Vector();
        path = normalizePath(path);
        if(path.startsWith(mountPoint)) {
            String list = path.substring(mountPoint.length());
            if((path.endsWith(".pls") && supportPLS) || (path.endsWith(".m3u") && supportM3U)) {
                return(PlayList.list(mapPath + list, false));
            }
            File file[] = new File(mapPath + list).listFiles();
            if(file == null)
                return null;
            for(int i = 0; i < file.length; i++) {
                String name = file[i].getName();
                if(file[i].isDirectory() || (name.endsWith(".pls") && supportPLS) || (name.endsWith(".m3u") && supportM3U))
                    name += FileSystem.pathSeparatorChar;
                ret.add(name);
            } 
        } else if(mountPoint.startsWith(path)) {
            String right = mountPoint.substring(path.length()+1);
            if(right.indexOf(FileSystem.pathSeparatorChar) > 0)
                right = right.substring(0, right.indexOf(FileSystem.pathSeparatorChar));
            ret.add(right + FileSystem.pathSeparatorChar);
        } else {
            return null;
        }
        return ret;
    }
     
    protected static String normalizePath(String path) {
        path = path.trim();
        if(path.endsWith(FileSystem.pathSeparatorChar))
            path = path.substring(0, path.length() - 1);
        if(!path.startsWith(FileSystem.pathSeparatorChar))
            path = FileSystem.pathSeparatorChar + path;
        return(path);
    }        
}
